@@ -26,8 +26,11 @@ enum {
 struct GLRenderToTexture {
   GLuint m_framebufferName;
   GLuint m_depthrenderbuffer;
+  GLuint m_renderTextureId;
   bool m_initialized;
   int m_renderTextureType;
+  int m_width;
+  int m_height;
 
  public:
   GLRenderToTexture();
